@@ -2,6 +2,7 @@
 from io import StringIO
 import pandas as pd 
 import numpy as np
+import operator
 import math
 import os
 
@@ -41,7 +42,11 @@ def analyse_df(df):
             uniques.remove(np.nan)
         unicount = len(uniques)
         unipercent = round(100*unicount/records,1)
- 
+        mode = df[name].mode(dropna=False)[0]
+        #if (type(mode) == float) & np.isnan(mode):
+        #    mode = "NaN" 
+        if mode != mode:
+            mode = "NaN" 
         #valtype = infer_type(str(type(df.loc[1,name])), unicount, uniques)
         valtype = infer_type_2( df.loc[:,name], 0, unicount, uniques)
 
@@ -68,6 +73,7 @@ def analyse_df(df):
         values_to_add = {
             'Name':name, 
             'Type':valtype,
+            'Mode':mode,
             'Unique Vals':unicount, 
             'Unique':unipercent, 
             'Nulls':napercent, 
@@ -102,10 +108,11 @@ def analyse_file_in_chunks(path_to_file):
 
 ########################################################################################
 def generate_final_summary(temp, total_chunks):
-    rez = pd.DataFrame(columns=('Name', 'Type', 'Unique Vals', 'Unique', 'Nulls', 'Min', 'Mean', 'Max'))
+    rez = pd.DataFrame(columns=('Name', 'Mode', 'Type', 'Unique Vals', 'Unique', 'Nulls', 'Min', 'Mean', 'Max'))
     for name in temp.keys():
         col = temp[name]
         total = col['nulls'] + col['nonnulls']
+        mode = max(col['val_counts'].items(), key=operator.itemgetter(1))[0]
         unicount = col['uniques'].estimate()
         if unicount > total:
             uniprop = 1.0
@@ -120,6 +127,7 @@ def generate_final_summary(temp, total_chunks):
             themean = col['mean'] 
         values_to_add = {
             'Name':name, 
+            'Mode':mode,
             'Type': col['type'],
             'Unique Vals':unicount,
             'Unique':unipercent,
@@ -131,6 +139,22 @@ def generate_final_summary(temp, total_chunks):
         rez = rez.append(values_to_add, ignore_index=True)
     return rez
 
+########################################################################################
+def clean_dict(df, col):
+    temp = df[col].value_counts(dropna=False)
+    indeces = temp.index
+    newie = []
+    for i in indeces:
+        if np.isnan(i):
+            newie.append("NaN")
+        else:
+            newie.append(i)
+    temp.index = newie
+    return temp.to_dict()
+
+def combine_dicts(a, b, op=operator.add):
+    return {**a, **b, **{k: op(a[k], b[k]) for k in a.keys() & b}}
+
 ########################################################################################
 def update_temp_summary(temp, df, startpoint):
     colnames = df.columns
@@ -140,13 +164,14 @@ def update_temp_summary(temp, df, startpoint):
         if name in temp:
             rez = temp[name]
         else: 
-            rez = { "type":[], "sum":0, "mean":np.nan, 
+            rez = { "type":[], "val_counts":{}, "sum":0, "mean":np.nan, 
                     "min":np.nan, "max":np.nan, 
                     "uniques":FMEstimator(), "nulls":0, 
                     "nonnulls":0
                    }
         nacount = len(df[df[name].isna()])
         nonnulls = len(df) - nacount
+        val_counts = clean_dict(df, name)
         uniques = df[name].unique().tolist()
         if np.nan in uniques :
             uniques.remove(np.nan)
@@ -189,6 +214,7 @@ def update_temp_summary(temp, df, startpoint):
             rez['max'] = themax
         rez['uniques'].update_all(uniques)
         #rez['uniques'] += uniprop
+        rez['val_counts'] = combine_dicts(rez['val_counts'], val_counts)
         rez['nonnulls'] = rez['nonnulls'] + nonnulls
         temp[name] = rez
     return temp
@@ -338,14 +364,15 @@ def print_latex(summary):
     print(" \\begin{center}")
     print("   \\caption{Data Summary Table}")
     print("   \\label{tab:table1}")
-    print("   \\begin{tabular}{l|l|r|r|r|r} ")
-    print("    \\textbf{Name} & \\textbf{Type} & \\textbf{Unique Vals \%} & \\textbf{Nulls \%} & \\textbf{Min} & \\textbf{Mean} & \\textbf{Max}\\\\")
+    print("   \\begin{tabular}{l|l|r|r|r|r|r} ")
+    print("    \\textbf{Name} & \\textbf{Type} & \\textbf{Unique Vals \%} & \\textbf{Nulls \%} & \\textbf{Mode} & \\textbf{Min} & \\textbf{Mean} & \\textbf{Max}\\\\")
     print("      \\hline")
     for i in range(len(summary)):
         print("      ", summary.loc[i,"Name"], 
               "&", summary.loc[i,"Type"], 
               "&", summary.loc[i,"Unique Vals"], "%" 
               "&", summary.loc[i,"Nulls"], "%" 
+              "&", summary.loc[i,"Mode"], 
               "&", summary.loc[i,"Min"], 
               "&", summary.loc[i,"Mean"], 
               "&", summary.loc[i,"Max"], "\\\\")
@@ -429,6 +456,22 @@ def print_csv(s):
     s.to_csv(output, index=False)
     print(output.getvalue())
 
+########################################################################################
+def get_padded_val2(val, spacer):
+    filler = ""
+    if spacer > 10:
+        filler = (get_spaces(spacer - 10))
+    if type(val) == int:
+        return filler + get_padded_number(val) 
+    if type(val) == float:
+        return filler + get_padded_number(val) 
+    else:
+        printval = str(val)
+        return (get_spaces(spacer - len(printval)) + printval + " ")
+
+def get_padded_val(val):
+    return str(val) + " - " + str(type(val))
+
 ########################################################################################
 def print_markdown(s):
     longest_name = max(s["Name"].apply(lambda x: len_or_null(x)))
@@ -437,16 +480,25 @@ def print_markdown(s):
     else:
         name_spacer = 6
 
+    longest_mode = max(s["Mode"].apply(lambda x: len_or_null(str(x))))
+    if(longest_mode>10):
+        mode_spacer = longest_mode
+    else:
+        mode_spacer = 10
+
     print("| Name ", get_spaces(name_spacer-6), 
-        "| Type   | Unique Vals | Nulls   |  Min       |  Mean      |  Max       |", sep="")
+        "| Type   | Unique Vals | Nulls   | Mode ", get_spaces(mode_spacer-4),
+        "|  Min       |  Mean      |  Max       |", sep="")
     print("| ---- ", get_spaces(name_spacer-6), 
-        "| ------ | ----------- | ------- |  ---       |  ----      |  ---       |", sep="")
+        "| ------ | ----------- | ------- | ---- ", get_spaces(mode_spacer-4),
+        "|  ---       |  ----      |  ---       |", sep="")
     for i in range(len(s)):
         print("| ", s.loc[i,"Name"], 
             get_spaces(name_spacer - len(s.loc[i,"Name"]) - 1 ), 
             "| ", s.loc[i,"Type"], get_type_spacer(s.loc[i,"Type"]),
             "|  ", get_padded_number(s.loc[i,"Unique Vals"]), 
             "| ", get_percent_spacer(s.loc[i,"Nulls"]), s.loc[i,"Nulls"],"% ", 
+            "| ", get_padded_val2(s.loc[i,"Mode"], mode_spacer), 
             "| ", get_padded_number(s.loc[i,"Min"]), 
             "| ", get_padded_number(s.loc[i,"Mean"]),
             "| ", get_padded_number(s.loc[i,"Max"]), "|", sep="")
